@@ -11,7 +11,7 @@ echo "Removing {arch} directories..."
 rm -rf "{arch}"
 rm -rf "curses/{arch}"
 echo "Generating ./configure ..."
-autoconf && rm -rf autom4te.cache
+autoconf2.13 && rm -rf autom4te.cache
 echo "Creating documentation..."
 cd doc && make &> /dev/null
 echo "Done."
