@@ -1,5 +1,5 @@
 (*  Wyrd -- a curses-based front-end for Remind
- *  Copyright (C) 2005 Paul Pelzl
+ *  Copyright (C) 2005, 2006 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -47,7 +47,7 @@ type command_t = | ScrollUp | ScrollDown | NextDay | PrevDay
                  | ViewAllReminders | ViewWeek | ViewMonth
                  | NextReminder | ViewKeybindings | CopyReminder
                  | PasteReminder | PasteReminderDialog 
-                 | CutReminder | Goto
+                 | CutReminder | Goto | QuickEvent
                  | NewGenReminder of int | NewGenReminderDialog of int
 
 type entry_operation_t = | EntryComplete | EntryBackspace | EntryExit
@@ -115,6 +115,8 @@ let description_12_hour = ref true
 let center_cursor = ref false
 (* "jump to" date syntax is big-endian? *)
 let goto_big_endian = ref true
+(* print week numbers? *)
+let number_weeks = ref false
 (* List of included rc files *)
 let included_rcfiles : (string list) ref = ref []
 
@@ -309,6 +311,7 @@ let commands_list = [
    ("paste_dialog"            , CommandOp PasteReminderDialog);
    ("scroll_description_up"   , CommandOp ScrollDescUp);
    ("scroll_description_down" , CommandOp ScrollDescDown);
+   ("quick_add"               , CommandOp QuickEvent);
    ("new_timed"               , CommandOp NewTimed);
    ("new_timed_dialog"        , CommandOp NewTimedDialog);
    ("new_untimed"             , CommandOp NewUntimed);
@@ -403,6 +406,7 @@ let parse_line line_stream =
          | [< 'Ident "magenta" >] -> Color.magenta
          | [< 'Ident "cyan" >]    -> Color.cyan
          | [< 'Ident "white" >]   -> Color.white
+         | [< 'Ident "default" >] -> ~- 1
          | [< >] ->
             config_failwith ("Expected a foreground color after \"set " ^ obj_str)
          end
@@ -417,6 +421,7 @@ let parse_line line_stream =
          | [< 'Ident "magenta" >] -> Color.magenta
          | [< 'Ident "cyan" >]    -> Color.cyan
          | [< 'Ident "white" >]   -> Color.white
+         | [< 'Ident "default" >] -> ~- 1
          | [< >] ->
             config_failwith ("Expected a background color after \"set " ^ obj_str)
          end
@@ -440,8 +445,12 @@ let parse_line line_stream =
       let bind_key key = 
          begin match line_stream with parser
          | [< 'Ident command_str >] ->
-            let command = operation_of_string command_str in
-            register_binding key command
+            begin try
+               let command = operation_of_string command_str in
+               register_binding key command
+            with Not_found ->
+               config_failwith ("Unrecognized command name \"" ^ command_str ^ "\"")
+            end
          | [< >] ->
             config_failwith ("Expected a command name after \"bind \"" ^ key ^ "\"")
          end
@@ -534,6 +543,8 @@ let parse_line line_stream =
          parse_set "center_cursor" center_cursor bool_of_string "Expected a boolean string after "
       | [< 'Ident "goto_big_endian" >] ->
          parse_set "goto_big_endian" goto_big_endian bool_of_string "Expected a boolean string after "
+      | [< 'Ident "number_weeks" >] ->
+         parse_set "number_weeks" number_weeks bool_of_string "Expected a boolean string after "
       | [< >] ->
          config_failwith ("Unmatched variable name after \"set\"")
       end
