@@ -1,5 +1,5 @@
 (*  Wyrd -- a curses-based front-end for Remind
- *  Copyright (C) 2005 Paul Pelzl
+ *  Copyright (C) 2005, 2006 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -27,26 +27,144 @@ exception Occurrence_not_found
 open Utility
 
 
+(* A record for an individual timed reminder, as read from
+ * the output of 'remind -s'. *)
+type timed_rem_t = {
+   tr_start      : float;
+   tr_end        : float;
+   tr_msg        : string;
+   tr_filename   : string;
+   tr_linenum    : string;
+   tr_has_weight : bool
+}
+
+
+(* A record for an individual untimed reminder, as read from
+ * the output of 'remind -s'. *)
+type untimed_rem_t = {
+   ur_start      : float;
+   ur_msg        : string;
+   ur_filename   : string;
+   ur_linenum    : string;
+   ur_has_weight : bool
+}
+
+
 (* Storage for a three-month window of reminders and
  * the calendar for the current month.
  * Makes it possible to handle edge effects of moving
  * from month to month without constantly calling 
- * rem(1) and regenerating calendar layouts. *)
+ * rem(1) and regenerating calendar layouts. 
+ *
+ * Timed reminders are stored in an array of lists,
+ * with each element of the array representing a different
+ * indentation level on the timetable.  Untimed reminders
+ * have no indentation, so they are simply stored in
+ * lists. *)
 type three_month_rem_t = {
    curr_timestamp : float;
-   prev_timed     : (float * float * string * string * string * bool) list array;
-   curr_timed     : (float * float * string * string * string * bool) list array;
-   next_timed     : (float * float * string * string * string * bool) list array;
-   all_timed      : (float * float * string * string * string * bool) list array;
-   prev_untimed   : (float * string * string * string * bool) list;
-   curr_untimed   : (float * string * string * string * bool) list;
-   next_untimed   : (float * string * string * string * bool) list;
-   all_untimed    : (float * string * string * string * bool) list;
+   prev_timed     : timed_rem_t list array;
+   curr_timed     : timed_rem_t list array;
+   next_timed     : timed_rem_t list array;
+   all_timed      : timed_rem_t list array;
+   prev_untimed   : untimed_rem_t list;
+   curr_untimed   : untimed_rem_t list;
+   next_untimed   : untimed_rem_t list;
+   all_untimed    : untimed_rem_t list;
    curr_counts    : int array;
-   curr_cal       : Cal.t
+   curr_cal       : Cal.t;
+   remind_error   : string
 }
 
 
+(* Get the starting timestamp and time record for a given month *)
+let month_start_of_tm tm =
+   let month_start_tm = {empty_tm with
+      Unix.tm_mon   = tm.Unix.tm_mon;
+      Unix.tm_year  = tm.Unix.tm_year;
+   } in
+   Unix.mktime month_start_tm
+
+
+
+(* Process information about a timed reminder, create a new reminder
+ * record for it, and store it in the timed reminders array at the
+ * proper indentation level.
+ *
+ * The indentation levels are determined by maintaining an array that indicates
+ * which levels have been used for each hourly timeslot.  As each reminder is
+ * processed, we look into the array to find the smallest indentation level
+ * available.  Complexity is approximately
+ * (number of reminders) * (average reminder duration).  Since indentation is
+ * determined a month at a time, there may be some minor discrepancies at
+ * month borders. *)
+let process_timed tm duration_s month_start_ts indentations partial_trem 
+timed =
+   let (f_rem_ts, _) = Unix.mktime tm in
+   let duration =
+      if duration_s = "*" then 0.0
+      else float_of_string duration_s
+   in
+   (* compute the indentation level *)
+   (* top_index and bottom_index provide the range of row indices into
+    * array indentations that are covered by this reminder *)
+   let top_index = 
+      try int_of_float ((f_rem_ts -. month_start_ts) /. 3600.0)
+      with _ -> 0
+   in
+   let bottom_index = 
+      try
+         let real_bottom_index =
+            let shift = 
+               if duration > 0.0 then
+                  (f_rem_ts +. (duration *. 60.0) -. month_start_ts) /. 3600.0
+               else
+                  (f_rem_ts +. (duration +. 60.0) -. month_start_ts) /. 3600.0
+            in
+            (* catch the edge effects when reminders end on hour boundaries *)
+            if shift = float_of_int (int_of_float shift) then
+               pred (int_of_float shift)
+            else
+               int_of_float shift
+         in
+         (* the bottom index could flow off of this month, in which case
+          * we truncate and hope everything works out *)
+         if real_bottom_index > pred (Array.length indentations) then
+            pred (Array.length indentations)
+         else
+            real_bottom_index
+      with _ -> top_index
+   in
+   (* locate the smallest free indentation level *)
+   let rec find_indent level =
+      if level < Array.length indentations.(0) then begin
+         let collision = ref false in
+         for i = top_index to bottom_index do
+            if indentations.(i).(level) then
+               collision := true
+            else
+               ()
+         done;
+         if !collision then
+            find_indent (succ level)
+         else begin
+            for i = top_index to bottom_index do
+               indentations.(i).(level) <- true
+            done;
+            level
+         end
+      end else
+         (* give up and default to maximum indentation *)
+         pred (Array.length indentations.(0))
+   in
+   let indent = find_indent 0 in 
+   let trem = {partial_trem with
+      tr_start      = f_rem_ts;
+      tr_end        = f_rem_ts +. (duration *. 60.);
+   } in
+   timed.(indent) <- trem :: timed.(indent)
+
+
 
 (* Obtain two lists of reminders for the month of the timestamp argument.
  * The first list is for timed reminders, the second is for untimed. 
@@ -69,24 +187,18 @@ let month_reminders timestamp =
    let rem_date_str = (string_of_tm_mon tm.Unix.tm_mon) ^ " " ^ 
                       (string_of_int tm.Unix.tm_mday) ^ " " ^
                       (string_of_int (tm.Unix.tm_year + 1900)) in
-   let remind_channel = Unix.open_process_in (!Rcfile.remind_command ^ " -s -l -g -b2 " ^ 
-   !Rcfile.reminders_file ^ " " ^ rem_date_str) in
+   let (remind_channel, remind_in_channel, remind_err_channel) = 
+      Unix.open_process_full (!Rcfile.remind_command ^ " -s -l -g -b2 " ^ 
+      !Rcfile.reminders_file ^ " " ^ rem_date_str) [||] 
+   in
+   (* check for Remind errors *)
+   let remind_err =
+      try input_line remind_err_channel
+      with End_of_file -> ""
+   in
    let num_indentations = 4 in
    let indentations = Array.make_matrix (24 * 31) num_indentations false in
-   let (month_start_ts, _) =
-      let temp = {
-         Unix.tm_sec   = 0;
-         Unix.tm_min   = 0;
-         Unix.tm_hour  = 0;
-         Unix.tm_mday  = 1;
-         Unix.tm_mon   = tm.Unix.tm_mon;
-         Unix.tm_year  = tm.Unix.tm_year;
-         Unix.tm_wday  = 0;
-         Unix.tm_yday  = 0;
-         Unix.tm_isdst = false
-      } in
-      Unix.mktime temp
-   in
+   let (month_start_ts, _) = month_start_of_tm tm in
    let timed = Array.make num_indentations [] in
    let rec build_lists untimed =
       try
@@ -106,86 +218,44 @@ let month_reminders timestamp =
                let year     = int_of_string date_arr.(0)
                and month    = int_of_string date_arr.(1)
                and day      = int_of_string date_arr.(2) in
-               let temp = {
-                  Unix.tm_sec   = 0;
-                  Unix.tm_min   = 0;
-                  Unix.tm_hour  = 0;
+               let temp = {empty_tm with
                   Unix.tm_mday  = day;
                   Unix.tm_mon   = pred month;
                   Unix.tm_year  = year - 1900;
-                  Unix.tm_wday  = 0;
-                  Unix.tm_yday  = 0;
-                  Unix.tm_isdst = false
                } in
                (* check whether this reminder is tagged 'nodisplay' *)
-               if not (Str.string_match nodisplay_regex tag 0) then begin
-                  let has_weight = not (Str.string_match noweight_regex tag 0) in
-                  if min_s = "*" then
-                     let (f_rem_ts, _) = Unix.mktime temp in
-                     build_lists ((f_rem_ts, msg, filename, line_num_s, has_weight) :: untimed)
-                  else begin
-                     let temp_with_min = {temp with Unix.tm_min = int_of_string min_s} in
-                     let (f_rem_ts, _) = Unix.mktime temp_with_min in
-                     let duration =
-                        if duration_s = "*" then 0.0
-                        else float_of_string duration_s
-                     in
-                     (* compute the indentation level *)
-                     (* top_index and bottom_index provide the range of row indices into
-                      * array indentations that are covered by this reminder *)
-                     let top_index = 
-                        try
-                           int_of_float ((f_rem_ts -. month_start_ts) /. 3600.0)
-                        with _ -> 0
-                     in
-                     let bottom_index = 
-                        try
-                           let real_bottom_index =
-                              let shift = (f_rem_ts +. (duration *. 60.0) -. month_start_ts) /. 3600.0 in
-                              (* catch the edge effects when reminders end on hour boundaries *)
-                              if shift = float_of_int (int_of_float shift) then
-                                 pred (int_of_float shift)
-                              else
-                                 int_of_float shift
-                           in
-                           (* the bottom index could flow off of this month, in which case
-                            * we truncate and hope everything works out *)
-                           if real_bottom_index > pred (Array.length indentations) then
-                              pred (Array.length indentations)
-                           else
-                              real_bottom_index
-                        with _ -> top_index
-                     in
-                     (* locate the smallest free indentation level *)
-                     let rec find_indent level =
-                        if level < Array.length indentations.(0) then begin
-                           let collision = ref false in
-                           for i = top_index to bottom_index do
-                              if indentations.(i).(level) then
-                                 collision := true
-                              else
-                                 ()
-                           done;
-                           if !collision then
-                              find_indent (succ level)
-                           else begin
-                              for i = top_index to bottom_index do
-                                 indentations.(i).(level) <- true
-                              done;
-                              level
-                           end
-                        end else
-                           (* give up and default to maximum indentation *)
-                           pred (Array.length indentations.(0))
-                     in
-                     let indent = find_indent 0 in
-                     timed.(indent) <- (f_rem_ts, f_rem_ts +. (duration *. 60.), msg, 
-                                       filename, line_num_s, has_weight) :: timed.(indent);
-                     build_lists untimed
-                  end
-               end else
+               if (Str.string_match nodisplay_regex tag 0) then
                   (* skip this reminder due to a 'nodisplay' tag *)
                   build_lists untimed
+               else begin
+                  let has_weight = not (Str.string_match noweight_regex tag 0) in
+                  if min_s = "*" then
+                     (* if minutes are not provided, this must be an untimed reminder *)
+                     let (f_rem_ts, _) = Unix.mktime temp in
+                     let urem = {
+                        ur_start      = f_rem_ts;
+                        ur_msg        = msg;
+                        ur_filename   = filename;
+                        ur_linenum    = line_num_s;
+                        ur_has_weight = has_weight
+                     } in
+                     build_lists (urem :: untimed)
+                  else begin
+                     (* if minutes are provided, this must be a timed reminder *)
+                     let temp_with_min = {temp with Unix.tm_min = int_of_string min_s} in
+                     let partial_trem = {
+                        tr_start      = 0.0;  (* still needs to be filled in *)
+                        tr_end        = 0.0;  (* still needs to be filled in *)
+                        tr_msg        = msg;
+                        tr_filename   = filename;
+                        tr_linenum    = line_num_s;
+                        tr_has_weight = has_weight
+                     } in
+                     process_timed temp_with_min duration_s month_start_ts indentations
+                     partial_trem timed;
+                     build_lists untimed
+                  end
+               end
             end else
                (* if there was no rem_regex match, continue with next line *)
                build_lists untimed
@@ -194,11 +264,13 @@ let month_reminders timestamp =
             build_lists untimed
       with
       | End_of_file ->
-           let _ = Unix.close_process_in remind_channel in
+           let _ = Unix.close_process_full (remind_channel, 
+              remind_in_channel, remind_err_channel) 
+           in
            for i = 0 to pred (Array.length timed) do
               timed.(i) <- List.rev timed.(i)
            done;
-           (timed, List.rev untimed)
+           (remind_err, timed, List.rev untimed)
       | _ ->
            (* if there's an error in regexp matching or string coersion,
             * just drop that reminder and go to the next line *)
@@ -220,11 +292,11 @@ let count_reminders month_start_tm timed untimed =
       else
          ()
    in
-   let count_timed (start, _, _, _, _, has_weight) =
-      count_rems start has_weight
+   let count_timed rem =
+      count_rems rem.tr_start rem.tr_has_weight
    in
-   let count_untimed (start, _, _, _, has_weight) =
-      count_rems start has_weight
+   let count_untimed rem =
+      count_rems rem.ur_start rem.ur_has_weight
    in
    Array.iter (List.iter count_timed) timed;
    List.iter count_untimed untimed;
@@ -267,12 +339,12 @@ let count_busy_hours month_start_tm timed untimed =
       else
          ()
    in
-   let count_timed (start, stop, _, _, _, has_weight) =
-      count_hours start stop has_weight
+   let count_timed rem =
+      count_hours rem.tr_start rem.tr_end rem.tr_has_weight
    in
-   let count_untimed (start, _, _, _, has_weight) =
-      let stop = start +. (!Rcfile.untimed_duration *. 60.) in
-      count_hours start stop has_weight
+   let count_untimed rem =
+      let stop = rem.ur_start +. (!Rcfile.untimed_duration *. 60.) in
+      count_hours rem.ur_start stop rem.ur_has_weight
    in
    Array.iter (List.iter count_timed) timed;
    List.iter count_untimed untimed;
@@ -295,14 +367,10 @@ let count_busy month_tm timed untimed =
 
 (* comparison functions for sorting reminders chronologically *)
 let cmp_timed rem_a rem_b =
-   let (ts_a, _, _, _, _, _) = rem_a
-   and (ts_b, _, _, _, _, _) = rem_b in
-   compare ts_a ts_b
+   compare rem_a.tr_start rem_b.tr_start
 
 let cmp_untimed rem_a rem_b =
-   let (ts_a, _, _, _, _) = rem_a
-   and (ts_b, _, _, _, _) = rem_b in
-   compare ts_a ts_b
+   compare rem_a.ur_start rem_b.ur_start
 
 
 (* take an array of timed reminder lists and merge them into
@@ -336,14 +404,22 @@ let create_three_month timestamp =
    } in
    let (prev_ts, _) = Unix.mktime temp_prev
    and (next_ts, _) = Unix.mktime temp_next in
-   let (pt, pu) = month_reminders prev_ts in
-   let (ct, cu) = month_reminders curr_ts in
-   let (nt, nu) = month_reminders next_ts in 
+   let (pre, pt, pu) = month_reminders prev_ts in
+   let (cre, ct, cu) = month_reminders curr_ts in
+   let (nre, nt, nu) = month_reminders next_ts in 
    let at = Array.make (Array.length pt) [] in
    for i = 0 to pred (Array.length at) do
       at.(i) <- safe_append pt.(i) (safe_append ct.(i) nt.(i))
    done;
-   let au = safe_append cu (safe_append pu nu) in {
+   let err_str = 
+      if String.length pre > 0 then
+         pre
+      else if String.length cre > 0 then
+         cre
+      else
+         nre
+   in
+   let au = safe_append pu (safe_append cu nu) in {
       curr_timestamp = curr_ts;
       prev_timed     = pt;
       curr_timed     = ct;
@@ -354,7 +430,8 @@ let create_three_month timestamp =
       next_untimed   = nu;
       all_untimed    = au;
       curr_counts    = count_busy month_start_tm at au;
-      curr_cal       = Cal.make curr_ts !Rcfile.week_starts_monday
+      curr_cal       = Cal.make curr_ts !Rcfile.week_starts_monday;
+      remind_error   = err_str   
    }
 
 
@@ -370,7 +447,7 @@ let next_month reminders =
    } in
    let (new_curr_timestamp, temp1) = Unix.mktime temp1 in
    let (next_timestamp, temp2)     = Unix.mktime temp2 in
-   let (t, u) = month_reminders next_timestamp in 
+   let (re, t, u) = month_reminders next_timestamp in 
    let at = Array.make (Array.length t) [] in
    for i = 0 to pred (Array.length t) do
       at.(i) <- safe_append reminders.curr_timed.(i) 
@@ -387,7 +464,8 @@ let next_month reminders =
       next_untimed   = u;
       all_untimed    = au;
       curr_counts    = count_busy temp1 at au;
-      curr_cal       = Cal.make new_curr_timestamp !Rcfile.week_starts_monday
+      curr_cal       = Cal.make new_curr_timestamp !Rcfile.week_starts_monday;
+      remind_error   = re
    }
 
 
@@ -402,7 +480,7 @@ let prev_month reminders =
    } in
    let (new_curr_timestamp, temp1) = Unix.mktime temp1 in
    let (prev_timestamp, temp2)     = Unix.mktime temp2 in
-   let (t, u) = month_reminders prev_timestamp in
+   let (re, t, u) = month_reminders prev_timestamp in
    let at = Array.make (Array.length t) [] in
    for i = 0 to pred (Array.length t) do
       at.(i) <- safe_append t.(i) 
@@ -419,7 +497,8 @@ let prev_month reminders =
       next_untimed   = reminders.curr_untimed;
       all_untimed    = au;
       curr_counts    = count_busy temp1 at au;
-      curr_cal       = Cal.make new_curr_timestamp !Rcfile.week_starts_monday
+      curr_cal       = Cal.make new_curr_timestamp !Rcfile.week_starts_monday;
+      remind_error   = re
    }
 
 
@@ -476,10 +555,12 @@ let find_next msg_regex timestamp =
    let rem_date_str2 = (string_of_tm_mon tm2.Unix.tm_mon) ^ " " ^ 
                        (string_of_int tm2.Unix.tm_mday) ^ " " ^
                        (string_of_int (tm2.Unix.tm_year + 1900)) in
-   let remind_channel = Unix.open_process_in (!Rcfile.remind_command ^ " -n -s -b1 " ^ 
-   !Rcfile.reminders_file ^ " " ^ rem_date_str1 ^ " > " ^ Rcfile.tmpfile ^ " && " ^
-   !Rcfile.remind_command ^ " -n -s -b1 " ^ !Rcfile.reminders_file ^ " " ^ rem_date_str2 ^ " | cat " ^ 
-   Rcfile.tmpfile ^ " - | sort") in
+   let (remind_channel, remind_in_channel, remind_err_channel) = 
+      Unix.open_process_full (!Rcfile.remind_command ^ " -n -s -b1 " ^ 
+      !Rcfile.reminders_file ^ " " ^ rem_date_str1 ^ " > " ^ Rcfile.tmpfile ^ " && " ^
+      !Rcfile.remind_command ^ " -n -s -b1 " ^ !Rcfile.reminders_file ^ " " ^ rem_date_str2 ^ " | cat " ^ 
+      Rcfile.tmpfile ^ " - | sort") [||]
+   in
    let rec check_messages () =
       try
          let line = input_line remind_channel in
@@ -491,16 +572,11 @@ let find_next msg_regex timestamp =
             and tag   = Str.matched_group 4 line
             and min_s = Str.matched_group 5 line
             and msg   = Str.matched_group 6 line in
-            let temp = {
-               Unix.tm_sec   = 0;
+            let temp = {empty_tm with
                Unix.tm_min   = if min_s = "*" then 0 else (int_of_string min_s);
-               Unix.tm_hour  = 0;
                Unix.tm_mday  = day;
                Unix.tm_mon   = pred month;
                Unix.tm_year  = year - 1900;
-               Unix.tm_wday  = 0;
-               Unix.tm_yday  = 0;
-               Unix.tm_isdst = false
             } in
             let (ts, _) = Unix.mktime temp in
             if ts > timestamp then
@@ -508,7 +584,9 @@ let find_next msg_regex timestamp =
                   let _ = Str.search_forward msg_regex msg 0 in
                   (* only return the match if this value is not tagged 'nodisplay' *)
                   if not (Str.string_match nodisplay_regex tag 0) then
-                     let _ = Unix.close_process_in remind_channel in
+                     let _ = Unix.close_process_full (remind_channel,
+                        remind_in_channel, remind_err_channel)
+                     in
                      ts
                   else
                      check_messages ()
@@ -523,7 +601,9 @@ let find_next msg_regex timestamp =
             check_messages ()
       with
       | End_of_file ->
-         let _ = Unix.close_process_in remind_channel in
+         let _ = Unix.close_process_full (remind_channel, 
+            remind_in_channel, remind_err_channel) 
+         in
          raise Occurrence_not_found
       | Failure s ->
         (* if there's an error in string coersion, just drop that reminder and go
