@@ -1,5 +1,5 @@
 (*  Wyrd -- a curses-based front-end for Remind
- *  Copyright (C) 2005 Paul Pelzl
+ *  Copyright (C) 2005, 2006 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -22,8 +22,6 @@
  *
  * miscellaneous helper functions that don't really fit elsewhere *)
 
-(* for some reason this is unrecognized if I leave it in txtin_parser.mly *)
-exception Txtin_error of string
 
 exception String_of_tm_mon_failure of string
 exception String_of_tm_wday_failure of string
@@ -186,4 +184,31 @@ let full_string_of_tm_wday i =
 
 
 
+(* it's useful to have an empty date record to save some typing *)
+let empty_tm = {
+   Unix.tm_sec   = 0;
+   Unix.tm_min   = 0;
+   Unix.tm_hour  = 0;
+   Unix.tm_mday  = 1;
+   Unix.tm_mon   = 0;
+   Unix.tm_year  = 1900;
+   Unix.tm_wday  = 0;
+   Unix.tm_yday  = 0;
+   Unix.tm_isdst = false
+}
+
+
+
+(* strip whitespace *)
+let strip s =
+   (* any amount of whitespace, followed by a non-whitespace char, 
+    * followed by any number of chars, followed by a non-whitespace char, 
+    * followed by any amount of whitespace *)
+   let re = Str.regexp "^[ \t]*\\([^ \t].*[^ \t]\\)[ \t]*$" in
+   Str.replace_first re "\\1" s
+
+
+
+
+
 (* arch-tag: DO_NOT_CHANGE_a87790db-2dd0-496c-9620-ed968f3253fd *)
